; Enemy routines

ENEMIES		= 10	; max of 10 enemies

					; 0 = inactive
SQUID 		= 1		; walks around
SQUID2		= 2		; squid that shoots
SKELETON	= 3		; chases player
MOLE        = 4     ; walks around
FAIRY		= 5		; flys and recovers player's health

;====================================================================

ClearEnemies:
 ld hl,Enemies
 ld de,8
 ld bc,(ENEMIES<<8)+0
ClearEnemiesLoop:
 ld (hl),c
 add hl,de
 djnz ClearEnemiesLoop
 ret

; input: bc = coords, a = number
; returns: z = not added, hl = pointer to special
AddEnemy:
 push af
 push bc
 ld hl,Enemies
 ld de,8
 ld b,ENEMIES
AddEnemyFind:
 ld a,(hl)
 or a
 jr z,AddEnemyAdd
 add hl,de
 djnz AddEnemyFind
 pop bc
 pop af
 sub a				; clear zero flag
 ret
AddEnemyAdd:
 pop bc
 pop af
 ld (hl),a			; number
 inc hl
 ld (hl),c			; y coord
 inc hl
 ld (hl),b			; x coord
 inc hl
 ld (hl),1			; dir to down
 inc hl
 ld (hl),0			; reset anim flag
 inc hl
 ld ix,EnemyStartingLife
 dec a
 ld (_@AddEnemyGetLife),a
_@AddEnemyGetLife =$+2
 ld a,(ix+0)
 ld (hl),a 			; starting life
 inc hl
 ld (hl),0			; first special byte to 0
 or a				; set zero flag
 ret

; starts with enemy 1
EnemyStartingLife:
 .db 2, 2, 3, 3, 50

UpdateEnemies:
 ld hl,Enemies
 ld b,ENEMIES
UpdateEnemiesLoop:
 ld a,(hl)
 or a
 jr z,UpdateEnemiesNext
 ld (_@EnemyHitPlayerType),a
 push bc
 push hl
 inc hl
 push hl
 ex de,hl
 ld hl,EnemyUpdateRoutines
 ld b,0
 dec a
 add a,a
 ld c,a
 add hl,bc
 ld a,(hl)
 inc hl
 ld h,(hl)
 ld l,a
 ld (_@UpdateEnemyRoutine),hl
 ex de,hl
 ld (_@UpdateEnemyGetCoords),hl
_@UpdateEnemyRoutine =$+1
 call $ffff 
 pop hl
 call GrayLoadDrawSprite 
 ld a,(PlayerJump)
 or a
 jr nz,UpdateEnemiesSkip
_@UpdateEnemyGetCoords =$+2
 ld bc,($ffff)
 ld de,(PlayerY)
 call CheckCollision
 call nc,EnemyHitPlayer  
UpdateEnemiesSkip:
 pop hl
 pop bc
UpdateEnemiesNext:
 ld de,8
 add hl,de
 djnz UpdateEnemiesLoop 
 ret

; a = enemy type
EnemyHitPlayer:
 ld a,0
_@EnemyHitPlayerType =$-1
 cp 5
 jp nz,HurtPlayer
 ld a,(PlayerMaxLife)
 ld hl,PlayerLife
 cp (hl)
 ret z
 ld (hl),a
 call RandomNumber
 and %00000011
 add a,33
 jp PrintString

; bc = coords to damage, a = amount of damage
; returns: carry set = enemy hit
; destroys: af, de, hl
DamageEnemies:
 ld (_@HurtEnemyAmount),a
 ld a,$b7						; opcode for <OR A>
 ld (_@DamageEnemiesReturn),a
 ld hl,Enemies
 ld a,ENEMIES
DamageEnemiesLoop:
 ld (_@DamageEnemiesCounter),a
 ld a,(hl)
 or a
 call nz,DamageEnemiesTryHurt
 ld de,8
 add hl,de
_@DamageEnemiesCounter =$+1
 ld a,0
 dec a
 jr nz,DamageEnemiesLoop
_@DamageEnemiesReturn =$
 or a
 ret
DamageEnemiesTryHurt:
 push hl
 push bc
 inc hl
 ld e,(hl)
 inc hl
 ld d,(hl)
_@DamageEnemiesCheckRoutine =$+1
 call CheckCollision
 inc hl
 inc hl
 inc hl
 jr c,DamageEnemiesSkip
 call HurtEnemy
 ld a,$37						; opcode for <SCF>
 ld (_@DamageEnemiesReturn),a
DamageEnemiesSkip: 
 pop bc
 pop hl
 ret

; bc = coords to damage, a = amount of damage
; returns: carry set = enemy hit
; destroys: af, de, hl
DamageEnemiesPoint:
 ld hl,CheckPoint
 ld (_@DamageEnemiesCheckRoutine),hl
 call DamageEnemies
 ld hl,CheckCollision
 ld (_@DamageEnemiesCheckRoutine),hl
 ret

; hl = pointer to life of enemy, (_@HurtEnemyAmount) = amount to damage
HurtEnemy:
 push hl
 dec hl
 dec hl
 dec hl
 ld d,(hl)
 dec hl
 ld e,(hl)
 push de
 ld hl,EnemyHitSpr 
 ld c,2
 call AddAnim
 pop bc
 pop hl
 ld a,(hl)
_@HurtEnemyAmount =$+1
 sub 0
 ld (hl),a
 dec a
 ret p
 ld de,-5
 add hl,de
 ld (hl),0
; enemy die animation goes here
 jp PlaceRandomItem

; starting at 1, this is the routine to use to update each enemy
; hl = pointer to coords of enemy
; returns: de = pointer to the sprite set to be used for drawing
EnemyUpdateRoutines:
 .dw UpdateSquid, UpdateSquidFire, UpdateSkeleton, UpdateMole
 .dw UpdateFairy

UpdateSquid:
 call UpdateEnemyDumb
 ld de,SquidSpr
 ret

UpdateSquidFire:
 push hl
 ld e,(hl)
 inc hl
 ld d,(hl)
 inc hl
 call RandomNumber
 cp 8
 ld a,(hl)
 call c,AddEnemyProj
 pop hl
 call UpdateEnemyDumb
 ld de,SquidSpr
 ret

UpdateSkeleton:
 call UpdateEnemyChase
 call UpdateEnemyMoveSlow
 ld de,SkeletonSpr
 ret

UpdateMole:
 call UpdateEnemyDumb
 ld de,MoleSpr
 ret

UpdateFairy:
 ld a,(Frame)
 rra
 jr nc,UpdateFairyNext
 and %00000100
 jr z,UpdateFairyDown
 inc (hl)
 jr UpdateFairyNext
UpdateFairyDown:
 dec (hl)
UpdateFairyNext:
 inc hl
 inc hl
 ld (hl),0
 inc hl
 inc hl
 ld a,(PlayerLife)
 ex de,hl 
 ld hl,PlayerMaxLife
 cp (hl)
 ex de,hl
 jr nz,UpdateFairyLive
 ld a,1
 ld (hl),a
 ld (_@HurtEnemyAmount),a
 push hl
 call HurtEnemy
 pop hl
UpdateFairyLive:
 ld (hl),50
 dec hl
 ld de,FairySpr 
 ld a,(Frame)
 and %00000011
 ret z
 jp UpdateAnim

; these enemies chase the player
; hl = pointer to coords
; returns: hl = pointer to dir
UpdateEnemyChase:
 ld c,(hl)
 inc hl
 ld b,(hl)
 inc hl
 ld de,(PlayerY)
 ld a,(Frame)
 and %00000100
 jr z,UpdateEnemyChaseVert 
 ld a,b
 sub d
 jr nc,UpdateEnemyChaseLeft
 cp -3
 jr nc,UpdateEnemyChaseVert
UpdateEnemyChaseRight:
 ld (hl),3   
 jr UpdateEnemyChaseDone
UpdateEnemyChaseLeft:
 ld (hl),2
 jr UpdateEnemyChaseDone
UpdateEnemyChaseVert:
 ld a,c
 sub e
 jr nc,UpdateEnemyChaseUp
 cp -3
 jr nc,UpdateEnemyChaseDone
UpdateEnemyChaseDown:
 ld (hl),1
 jr UpdateEnemyChaseDone
UpdateEnemyChaseUp:
 ld (hl),0
UpdateEnemyChaseDone:
 inc hl
 call UpdateAnim
 dec hl
 ret
 
; these guys just walk back and forth randomly (but in a good-looking way :)
; hl = pointer to coords
UpdateEnemyDumb:
 ld de,5
 add hl,de
 ld a,(hl)
 or a
 jr nz,UpdateEnemyDumbMove
 call RandomNumber
 and %00011111
 add a,16
 ld (hl),a
 dec hl
 dec hl
 dec hl
UpdateEnemyDumbNewDir:
 call RandomNumber
 and %00000011
 ld (hl),a
 ret
UpdateEnemyDumbMove:
 dec (hl)
 rra
 ret nc
 dec hl
 dec hl
 call UpdateAnim
 dec hl
 dec hl
 dec hl
 ld c,(hl)
 inc hl 
 ld b,(hl)
 inc hl
 jr UpdateEnemyMove

UpdateEnemyMoveSlow:
 ld a,(Frame)
 and %00000011
 ret z
 inc hl
 call UpdateAnim
 dec hl
; hl = pointer to dir
UpdateEnemyMove:
 ld a,(hl)
 dec hl
 or a
 jr z,UpdateEnemyMoveUp
 dec a
 jr z,UpdateEnemyMoveDown
 dec a
 jr z,UpdateEnemyMoveLeft
; hl = pointer to X coord
UpdateEnemyMoveRight:
 push hl
 call EnemyCheckEdgeRight
 pop hl   
 ret z
 call TryMoveRight
 jp c,EnemyCheckEdgeRightHit
 ret
UpdateEnemyMoveLeft:
 push hl
 call EnemyCheckEdgeLeft
 pop hl   
 ret z
 call TryMoveLeft
 jp c,EnemyCheckEdgeLeftHit
 ret
UpdateEnemyMoveUp:
 dec hl
 push hl
 call EnemyCheckEdgeUp
 pop hl   
 ret z
 call TryMoveUp
 jp c,EnemyCheckEdgeUpHit
 ret
UpdateEnemyMoveDown:
 dec hl
 push hl
 call EnemyCheckEdgeDown
 pop hl
 ret z
 call TryMoveDown
 jp c,EnemyCheckEdgeDownHit
 ret

; hl = pointer to x or y coord
; returns: zero set = collision
EnemyCheckEdgeUp:
 sub a
 cp (hl)
 ret nz
EnemyCheckEdgeUpHit:
 inc hl
 inc hl
 ld (hl),1
 ret
EnemyCheckEdgeDown:
 ld a,56
 cp (hl)
 ret nz
EnemyCheckEdgeDownHit:
 inc hl
 inc hl 
 ld (hl),3
 ret
EnemyCheckEdgeLeft:
 sub a
 cp (hl)
 ret nz
EnemyCheckEdgeLeftHit:
 inc hl
 ld (hl),0
 ret
EnemyCheckEdgeRight:
 ld a,120
 cp (hl)
 ret nz
EnemyCheckEdgeRightHit:
 inc hl
 ld (hl),2
 ret 

EnemyHitSpr:
	.db %00000000
	.db %01011010
	.db %00111100
	.db %01100110
	.db %01100110
	.db %00111100
	.db %01011010
	.db %00000000
	.db %00000000
	.db %01000010
	.db %00011000
	.db %00111100
	.db %00111100
	.db %00011000
	.db %01000010
	.db %00000000
	.db %00000000
	.db %01011010
	.db %00111100
	.db %01111110
	.db %01111110
	.db %00111100
	.db %01011010
	.db %00000000

EnemyHitSpr2:
	.db %00000000
	.db %00000000
	.db %00100100
	.db %00000000
	.db %00000000
	.db %00100100
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00100100
	.db %00011000
	.db %00011000
	.db %00100100
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00100100
	.db %00011000
	.db %00011000
	.db %00100100
	.db %00000000
	.db %00000000

#include "enemyproj.asm"

.end
